function varargout=plotmesh(self,varargin)
% FUNCTION plotmesh 
%   Plots the main mesh or the m-face meshes of an OrthMesh object.
%
% USAGE
%   Oh.plotmesh(...) or plotmesh(Oh, ...) where Oh is an OrthMesh object.
%   One can use key/value pairs to specify options. The keys will be the 
%   strings
%     'm' to specify the m-faces meshes to plot (0<=m<=Oh.d). Default m=Oh.d
%     'legend' set to true to plot the legend. False by default.
%   or any of the plotmesh options depending of Oh.d and m. Same key/pairs as
%     - patch   function for d=3 and m=3  ('orthotope' or 'simplicial' mesh)
%     - trimesh function for d=3 and m=2 ('simplicial' mesh)
%     - patch   function for d=3 and m=2 ('orthotope' mesh)
%     - line    function for d=3 and m=1 ('orthotope' or 'simplicial' mesh)
%     - scatter3   function for d=3 and m=0 ('simplicial' mesh)
%     - triplot function for d=2 and m=2 ('simplicial' mesh)
%     - patch   function for d=2 and m=2 ('orthotope' or 'simplicial' mesh)
%     - line    function for d=2 and m=1 ('orthotope' or 'simplicial' mesh)
%     - scatter function for d=2 and m=0 ('orthotope' or 'simplicial' mesh)
%     - line    function for d=1 and m=1 ('orthotope' or 'simplicial' mesh)
%     - scatter    function for d=1 and m=0 ('orthotope' or 'simplicial' mesh)
%
% SAMPLES
%   Oh=OrthMesh(3,15,'box',[-1 1;-2 2;-3 3],'type','simplex');
%   figure(1)
%   Oh.plotmesh('legend',true)
%   figure(2)
%   Oh.plotmesh('m',2,'legend',true,'edgecolor','w')
%
%    Copyright (C) 2016-2020 F. Cuvelier
  if self.d>3
    error('Unable to plot in dimension > 3!')
  end
  p = inputParser;
  p.KeepUnmatched=true; 
  p.PartialMatching=false;
  p.addParamValue('m',self.d,@isscalar);
  p.addParamValue('labels',[]);
  p.addParamValue('legend',false,@islogical);
  p.parse(varargin{:});
  R=p.Results;
  varargin=fc_tools.utils.deleteCellOptions(varargin,p.Parameters);
  assert( (R.m <= self.d) & (R.m>=0) )
  set(gcf(),'visible','off')
  if R.m==self.d
    h=self.Mesh.plotmesh(varargin{:});
    cLegend=self.Mesh.label;
  else
    idx=self.getFacesIndex(R.m);
    nF=length(self.Faces{idx});
    if isempty(R.labels)
      R.labels=1:nF;
    else
      assert(isempty(setdiff(R.labels,1:nF)),'Error with <labels> option')
    end
    pih=fc_tools.graphics.SetHoldOn(true);
    k=1;
    for i=R.labels
      h(k)=self.Faces{idx}(i).plotmesh(varargin{:});
      cLegend{k}=self.Faces{idx}(i).label;
      k=k+1;
    end
    fc_tools.graphics.RestoreHold(pih)
  end
  if R.legend
    if fc_tools.comp.isOctave()
      if ismac(),  figure(gcf());end %BUG with legend: error: set: "fontsize" must be greater than 0
      legend(h,cLegend)
    else
      if verLessThan('matlab','9.2.0') % R2017a
        legend(h,cLegend)
      else
        legend(h,cLegend,'autoupdate','off')
      end
    end
  end
  xlabel('x');
  if self.d==1,set(gca,'YTickLabel',{' '});end
  if self.d>1,ylabel('y');end
  if self.d>2,zlabel('z');end
  axis image
  set(gcf(),'visible','on')
  if self.d==3,view(3);end
  if nargout>=1,varargout{1}=h;end
  if nargout==2,varargout{2}=cLegend;end
end
