function varargout=buildmesh(geofile,N,varargin)
% FUNCTION fc_oogmsh.gmsh.buildmesh
%   Builds gmsh mesh file from a .geo file
%
% USAGE:
%   meshfile=fc_oogmsh.gmsh.buildmesh(geofile,N)
%   meshfile=fc_oogmsh.gmsh.buildmesh(geofile,N,key,value)
%
% INPUT:
%   geofile: name of the .geo file as string.
%   N: parameter used in the .geo file for mesh refinement.
%
%   optional parameters as key/value pairs:
%     'geodir': to specify the directory of the geofile.
%     'meshdir': to specify the directory where mesh wil be saved.
%     'meshfile': to specify the name of the mesh file.
%     'd': to specify space dimension 2(default) or 3.
%     'check': to check the generated mesh file if true. False as default false.
%   and all optional parameters of fc_oogmsh.gmsh.run
%
% SAMPLES:
%
% Copyright (C) 2017-2020 F. Cuvelier
  assert((round(N)==N) & (N>0))
  env=fc_oogmsh.environment();
  p = inputParser;
  p.KeepUnmatched=true; 
  p.addParamValue('geodir',env.geo_dir,@ischar); % select labels to draw
  p.addParamValue('meshdir',env.mesh_dir,@ischar);
  p.addParamValue('meshfile','',@ischar);
  p.addParamValue('d',2,@(x) ismember(x,[2,3]));
  p.addParamValue('check',false,@islogical);
  p.addParamValue('force',false,@islogical); 
  p.parse(varargin{:});
  R=p.Results;
  varargin=fc_tools.utils.deleteCellOptions(varargin,p.Parameters);
  if ~exist(R.meshdir)
    [success,message,messageid]=mkdir(R.meshdir);
    if ~success,error(message);end
  end
  %fullfiles=fc_oogmsh.find_geo([R.geodir,filesep,geofile]);
%    if isempty(fullfiles)
%      inputfile=fc_oogmsh.gmsh.findMeshOrGeofile(inputfile,R.geodir,R.meshdir);
%      assert(~isempty(fullfiles),'Unable to find geofile %s',geofile)
%    end
%    fullfile=fullfiles{1};
  fullfile=fc_oogmsh.gmsh.findMeshOrGeofile(geofile,R.geodir,R.meshdir);
  [pathstr,name,ext] = fileparts(fullfile);
  if isempty(R.meshfile) 
    R.meshfile=[R.meshdir,filesep,sprintf('%s-%d.msh',name,N)];
  else
    [pathstr,name,ext] = fileparts(R.meshfile);
    if isempty(pathstr), R.meshfile=[R.meshdir,filesep,name];end
    if isempty(ext) 
      R.meshfile=sprintf('%s-%d.msh',R.meshfile,N);
    else
      R.meshfile=[R.meshfile,ext];
    end 
  end
%    if exist(R.meshfile) && ~R.force
%      meshfile=R.meshfile;
%    else
    meshfile=fc_oogmsh.gmsh.run(fullfile,'d',R.d,'options',sprintf('-setnumber N %d',N), ...
                      'meshfile',R.meshfile,'force',R.force,varargin{:});
%  end
  if R.check
    [status,result]=fc_oogmsh.gmsh.checkmesh(meshfile);
    if status==0
      fprintf('[fc-oogmsh] Check mesh : OK\n')
    else
      fprintf('[fc-oogmsh] Check mesh : Failed\n')
    end
  end
  if nargout>=1, varargout{1}=meshfile;end
  if nargout==2, varargout{2}=fullfile;end  
end
